# JHC - Jabber Humane Clock.
# Copyright (C) 2007  Denis Nelubin aka Gelin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# http://gelin.ru
# mailto:den@gelin.ru
# xmpp:gelin@jabber.ru?message

""" This module provides Bot class which makes Jabber Bot
    functionality for application.
"""

import logging
import xmpp
import time

from jhc.dictionary import Dictionary
from jhc.timer import Timer

DEFAULT_RESOURCE = "jhc"

class Bot:
    """ Connects to Jabber server and serves user's time requests.
        Hides Jabber interaction inside.
    """

    def __init__(self, options):
        """ Takes options parameter which is dict of command line
            options. "jid", "password" and "database" are required attributes.
            JID can contains resource string.
        """
        self.log = logging.getLogger("jhc.bot.Bot")
        self.options = options
        self.dictionary = Dictionary(options.database)
        if not hasattr(self.options, "resource") or self.options.resource == None:
            self.options.resource = DEFAULT_RESOURCE
        self.log.info("options: %s", options)
        self.joinedConferences = []
        self.timer = Timer(self.options.time, self.options.interval, self.onTimer)

    def connect(self):
        """ Connects to and authenticates on jabber server.
            Sends initial time messages to all online contacts.
            Returns False if connection is failed.
        """
        self.log.info("connecting to %s:%s", self.options.server, self.options.port)
        self.connection = xmpp.Client(self.options.server, self.options.port, debug=[])
        if self.options.proxy:
            self.log.info("using proxy %s", self.options.proxy)
        result = self.connection.connect(proxy=self.options.proxy, secure=self.options.secure)
        self.log.info("connected via %s", result)
        if result == "tcp":
            self.log.warn("connection is unsecure")
        elif not result:
            self.log.fatal("connection failed")
            return False
        self.log.info("authenticating as %s", self.options.user)
        result = self.connection.auth(self.options.user, self.options.password,
                resource=self.options.resource)
        if not result:
            self.log.fatal("authentication failed")
            return False
        self.log.debug("sending initial presence")
        self.connection.sendInitPresence()
        self.log.info("connected successfully")
        self.log.debug("getting roster")
        self.roster = self.connection.getRoster()
        self.connection.Process(1)  #wait for roster receiving
        self.processRoster()
        self.connection.RegisterHandler('message', self.onMessage)
        self.connection.RegisterHandler('presence', self.onPresence)
        self.joinConferences()
        self.timer.start()
        self.timerEvent = False
        return True

    def serve(self):
        """ Starts serving cycle. Answers to incoming messages.
        """
        try:
            while True:
                result = self.connection.Process(1)
                if result == '0' and self.timerEvent:
                    self.onTimerWork()
        except KeyboardInterrupt:
            self.log.info("keyboard termination")

    def processRoster(self):
        """ Go through roster contacts and send to them all
            time message
        """
        if self.options.noRoster:
            return
        for jid in self.roster.getItems():
            item = self.roster.getItem(jid)
            subscription = item["subscription"]
            self.log.debug("%s subscription is %s", jid, subscription)
            if subscription and subscription.lower() in ("both", "to"):
                for resource in item["resources"]:
                    fullJid = self.makeFullJid(jid, resource)
                    self.sendMessage(fullJid)

    def joinConferences(self):
        """ Joins to conferences listed in parameters
        """
        for conf in self.options.conferences:
            self.joinConference(conf)

    def onMessage(self, connection, message):
        """ On message received handler
        """
        self.log.debug("got message\n%s", message)
        jid = message.getFrom()
        conf = self.getInvitation(message)
        if (conf):
            #invitation to conference received
            self.joinConference(conf)
        elif (jid.getResource() and message.getType() in ("chat", None)):
            #only messages from persons and only chats (no errors or groupchat)
            self.replyToMessage(message)
        elif message.getType() == "groupchat":
            #message from conference
            if not self.isJoined(jid):
                self.onGroupchatJoin(jid)
            elif message.getBody().startswith("/time") and not message.getTimestamp():
                #not answer to delayed messages
                self.sendGroupchatMessage(jid)

    def onPresence(self, connection, presence):
        """ On presence received handler
        """
        self.log.debug("got presence\n%s", presence)
        jid = presence.getFrom()
        ptype = presence.getType()
        if ptype == "subscribe":
            self.log.info("approving subscription to %s", jid)
            reply = xmpp.Presence(to=jid, typ="subscribed")
            self.connection.send(reply)
            self.log.info("request subscription from %s", jid)
            reply = xmpp.Presence(to=jid, typ="subscribe")
            self.connection.send(reply)
            self.sendMessage(jid)
        elif ptype == "subscribed":
            self.log.info("approved subscription from %s", jid)
        elif ptype in ("error", None):
            #check room entering
            for conf in self.options.conferences:
                if jid.bareMatch(conf):
                    if ptype == "error":
                        self.log.warn("failed to join to %s: %s", conf, presence.getError())
                    elif not self.isJoined(jid):
                        self.onGroupchatJoin(jid)

    def onGroupchatJoin(self, jid):
        """ Called internally when successfully joined to conference
        """
        self.log.info("joined to %s", jid)
        self.joinedConferences.append(jid.getStripped())
        self.sendGroupchatMessage(jid)

    def onTimer(self, args=[], kwargs={}):
        """ Called on timer tick. Just set flag that event come.
        """
        self.timerEvent = True

    def onTimerWork(self):
        """ Made real work on timer tick.
        """
        self.processRoster()
        for conf in self.joinedConferences:
            self.sendGroupchatMessage(conf)
        self.timerEvent = False

    def sendMessage(self, jid):
        """ Sends time message to specified jid
        """
        text = self.dictionary.getString(time.time())
        message = xmpp.Message(jid, text)
        self.log.info("sending to '%s' message '%s'", jid, text)
        self.log.debug(message)
        self.connection.send(message)

    def replyToMessage(self, message):
        """ Replies by time message to specified message
        """
        if message.getBody():
            message = self.makeReplyMessage(message)
            (jidFrom, jidTo) = (message.getFrom(), message.getTo())
            message.setBody(self.dictionary.getString(time.time()))
            self.log.info("repling to '%s' message '%s'", message.getTo(), message.getBody())
            self.log.debug(message)
            self.connection.send(message)
        else:
            self.log.debug("not repling to empty message")

    def sendGroupchatMessage(self, jid):
        """ Sends time message to specified room jid
        """
        jid = xmpp.JID(jid).getStripped()
        text = self.dictionary.getString(time.time())
        message = xmpp.Message(jid, text, typ="groupchat")
        self.log.info("sending to '%s' groupchat message '%s'", jid, text)
        self.log.debug(message)
        self.connection.send(message)

    def joinConference(self, jid):
        """ Joins to specified conference
        """
        jid = xmpp.JID(jid)
        if not jid.getResource():
            jid.setResource(self.options.defaultNick)
        listed = False
        for conf in self.options.conferences:
            if jid.bareMatch(conf):
                listed = True
        if not listed:
            self.options.conferences.append(str(jid))
        self.log.info("joining to %s", jid)
        presence = xmpp.Presence(to=jid)
        self.connection.send(presence)

    def makeReplyMessage(self, message):
        """ Constructs reply message
        """
        #message = message.buildReply(reply)    #doesn't copy message type
        reply = xmpp.Message(to=message.getFrom(),
                             typ=message.getType(),
                             subject=message.getSubject(),
                             frm=message.getTo())
        thread = message.getThread()
        if thread:
            reply.setThread(message.getThread())
        return reply

    def makeFullJid(self, bareJid, resource):
        """ Makes full JID from provided bare JID and resource
        """
        jid = xmpp.JID(bareJid)
        jid.setResource(resource)
        return jid

    def isJoined(self, jid):
        """ Checks is we were joined to specified conference
        """
        return xmpp.JID(jid).getStripped() in self.joinedConferences

    def getInvitation(self, message):
        """ Checks if specified message is inivitation to groupchat and
            returns room JID or None
        """
        result = None
        #check as groupchat
        node = message.getTag("x", namespace="jabber:x:conference")
        if node:
            result = node.getAttr("jid")
        else:
            #check as MUC
            node = message.getTag("x", namespace="http://jabber.org/protocol/muc#user")
            if node and node.getTag("invite"):
                result = message.getFrom()
        if result:
            self.log.info("got invitation to %s", result)
        return result