# JHC - Jabber Humane Clock.
# Copyright (C) 2007  Denis Nelubin aka Gelin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# http://gelin.ru
# mailto:den@gelin.ru
# xmpp:gelin@jabber.ru?message

""" Module with functions for command line interface used in scripts
"""

import logging
import optparse
import sys
import re
import xmpp
import types

#posible options definition
OPTIONS = {
"database-file":    optparse.make_option(
                    "-f", "--database-file",
                    dest="database",
                    help="file with SQLite database with time strings",
                    metavar="FILE"),
"cli-time":         optparse.make_option(
                    "-t", "--time",
                    dest="time",
                    help="display message for time in format HH:MM",
                    metavar="HH:MM"),
"jid":              optparse.make_option(
                    "-j", "--jid",
                    dest="jid",
                    help="JID which is used to connect to Jabber server",
                    metavar="JID"),
"password":         optparse.make_option(
                    "-p", "--password",
                    dest="password",
                    help="password which is used to connect to Jabber server",
                    metavar="PASSWORD"),
"server":           optparse.make_option(
                    "--server",
                    dest="server",
                    help="Jabber server host (if differs from JID domain part)",
                    metavar="HOST"),
"port":             optparse.make_option(
                    "--port",
                    dest="port", type="int", default=5222,
                    help="Jabber server port (if non-standard)",
                    metavar="PORT"),
"proxy-host":       optparse.make_option(
                    "--proxy-host",
                    dest="proxyHost",
                    help="proxy host (if need)",
                    metavar="HOST"),
"proxy-port":       optparse.make_option(
                    "--proxy-port",
                    dest="proxyPort", type="int",
                    help="proxy port (if need)",
                    metavar="PORT"),
"proxy-user":       optparse.make_option(
                    "--proxy-user",
                    dest="proxyUser",
                    help="proxy username (if need)",
                    metavar="USER"),
"proxy-password":   optparse.make_option(
                    "--proxy-password",
                    dest="proxyPassword",
                    help="proxy password (if need)",
                    metavar="PASSWORD"),
"secure":           optparse.make_option(
                    "--secure",
                    dest="secure", choices=["1", "0"],
                    help="force to enable (1) or disable (0) SSL/TLS encryption",
                    metavar="1|0"),
"user":             optparse.make_option(
                    "--user",
                    dest="user",
                    help="user name (if differs from JID node part)",
                    metavar="USER"),
"join-conference":  optparse.make_option(
                    "-c", "--join-conference",
                    action="append",
                    dest="conferences",
                    help="join conference on startup (can use many times)",
                    metavar="ROOM@MUC/NICK"),
"default-nick":     optparse.make_option(
                    "--default-nick",
                    dest="defaultNick", default="timebot",
                    help="nick which is used when bot is invited to conference (default: '%default')",
                    metavar="NICK"),
"no-roster":        optparse.make_option(
                    "--no-roster",
                    action="store_true",
                    dest="noRoster", default=False,
                    help="do not send time message to JIDs from roster"),
"bot-time":         optparse.make_option(
                    "-t", "--time",
                    dest="time", default="00:00",
                    help="start to send time message from specified time (default: '%default')",
                    metavar="HH:MM"),
"interval":         optparse.make_option(
                    "-i", "--interval",
                    dest="interval", type="int", default=30,
                    help="interval to send time message in minutes (default: '%default')",
                    metavar="MINUTES"),
}

#option group titles
GROUPS = {
"database":         "time string database options",
"jabber":           "connection to Jabber server options",
"proxy":            "proxy options",
"behaviour":        "bot behaviour options",
}

#common usage message
USAGE = "%prog options [config_file]"
#common description message
DESCRIPTION = """you can put options to command line or write them into
configuration file"""


class BaseParser:
    """ Base class with common methods used in command line option parsing
    """

    def __init__(self):
        #logger instance
        self.log = logging.getLogger("jhc.cli")

    def createParser(self, optionNames):
        """ Creates options parser for specified options list
        """
        self.parser = optparse.OptionParser(usage=USAGE,
                                            description=DESCRIPTION)
        for name in optionNames:
            self.parser.add_option(OPTIONS[name])

    def parseRecursive(self, args=None):
        """ Parses options recursive, reading all configuration
            files, specified and positioned parameters.
            Returns only options, without positioned args.
        """
        (options, posArgs) = self.parser.parse_args(args)
        for file in posArgs:
            fileArgs = self.readFile(file)
            fileOptions = self.parseRecursive(fileArgs)
            options = self.mergeOptions(fileOptions, options)
        return options

    def mergeOptions(self, options1, options2):
        """ Merges options1 and options2. Values from options2 overrides
            values from options1.
        """
        for option in self.parser._get_all_options():
            if option.dest:
                value = getattr(options2, option.dest)
                if value != None and value != option.default:
                    if (option.action == "append"):
                        options1.ensure_value(option.dest, []).extend(value)
                    else:
                        setattr(options1, option.dest, value)
        return options1

    def parseTime(self, timeString):
        """ Splits string in HH:MM format and returns tuple of two integers
        """
        time = timeString.split(":")
        return (int(time[0]), int(time[1]))

    def getParamsFromJid(self, options):
        """ flils user and server names and resource from provided JID.
            Don't override values if they are already defined.
        """
        jid = xmpp.JID(options.jid)
        if not hasattr(options, "user") or not options.user:
            options.user = jid.getNode()
        if not hasattr(options, "server") or not options.server:
            options.server = jid.getDomain()
        if not hasattr(options, "resource") or not options.resource:
            options.resource = jid.getResource()

    def readFile(self, filename):
        """ Reads specified file, removes comments and returns file content as
            list of arguments
        """
        result = []
        self.log.info("reading configuration from %s", filename)
        try:
            file = open(filename, "r")
            content = file.read()
            content = self.filterFile(content)
            result = content.split()
        except IOError:
            self.log.warn("can't read file %s", filename)
        return result

    def filterFile(self, content):
        """ Removes comments from config file content
        """
        return re.sub(r"#[^\n]*[\n$]", " ", content)


class CliParser(BaseParser):
    """ Class which parse options for jhc-cli.py script
    """

    def __init__(self):
        BaseParser.__init__(self)
        self.createParser(["database-file",
                           "cli-time"])

    def getOptions(self, args=None):
        """ Parses command line parameters for jhc-cli.py script
        """
        options = self.parseRecursive(args)
        if options.time:
            options.time = self.parseTime(options.time)
        if not options.database:
            self.parser.error("specify value for --database-file key")
        return options


class BotParser(BaseParser):
    """ Class which parse options for jhc-bot.py script
    """

    def __init__(self):
        BaseParser.__init__(self)
        self.createParser([])
        group = optparse.OptionGroup(self.parser, GROUPS["database"])
        group.add_option(OPTIONS["database-file"])
        self.parser.add_option_group(group)
        group = optparse.OptionGroup(self.parser, GROUPS["jabber"])
        group.add_option(OPTIONS["jid"])
        group.add_option(OPTIONS["password"])
        group.add_option(OPTIONS["user"])
        group.add_option(OPTIONS["server"])
        group.add_option(OPTIONS["port"])
        group.add_option(OPTIONS["secure"])
        self.parser.add_option_group(group)
        group = optparse.OptionGroup(self.parser, GROUPS["behaviour"])
        group.add_option(OPTIONS["join-conference"])
        group.add_option(OPTIONS["default-nick"])
        group.add_option(OPTIONS["no-roster"])
        group.add_option(OPTIONS["bot-time"])
        group.add_option(OPTIONS["interval"])
        self.parser.add_option_group(group)
        group = optparse.OptionGroup(self.parser, GROUPS["proxy"])
        group.add_option(OPTIONS["proxy-host"])
        group.add_option(OPTIONS["proxy-port"])
        group.add_option(OPTIONS["proxy-user"])
        group.add_option(OPTIONS["proxy-password"])
        self.parser.add_option_group(group)

    def getOptions(self, args=None):
        """ Parses command line parameters for jhc-bot.py script
        """
        options = self.parseRecursive(args)
        if options.jid:
            self.getParamsFromJid(options)
        if options.time:
            options.time = self.parseTime(options.time)
        options.proxy = {}
        if options.proxyHost:
            options.proxy["host"] = options.proxyHost
        if options.proxyPort:
            options.proxy["port"] = options.proxyPort
        if options.proxyUser:
            options.proxy["user"] = options.proxyUser
        if options.proxyPassword:
            options.proxy["password"] = options.proxyPassword
        if not options.database:
            self.parser.error("specify value for --database-file key")
        if not options.user:
            self.parser.error("specify value for --jid or --user key")
        if not options.server:
            self.parser.error("specify value for --jid or --server key")
        if not options.password:
            self.parser.error("specify value for --password key")
        return options