"""Tests for jhc.cli."""

import unittest
import optparse

import jhc.cli

class CliTest(unittest.TestCase):

    def testCliOptionsShort(self):
        args = "-f file.db -t 01:02"
        parser = jhc.cli.CliParser()
        opts = parser.getOptions(args.split())
        self.assertEqual("file.db", opts.database)
        self.assertEqual((1, 2), opts.time)

    def testCliOptionsLong(self):
        args = "--database-file file.db --time 01:02"
        parser = jhc.cli.CliParser()
        opts = parser.getOptions(args.split())
        self.assertEqual("file.db", opts.database)
        self.assertEqual((1, 2), opts.time)

    def testBotOptionsShort(self):
        args = "-f file.db -j user@server/resource -p pass -c conf1 -c conf2"
        parser = jhc.cli.BotParser()
        opts = parser.getOptions(args.split())
        self.assertEqual("file.db", opts.database)
        self.assertEqual("user@server/resource", opts.jid)
        self.assertEqual("user", opts.user)
        self.assertEqual("server", opts.server)
        self.assertEqual("resource", opts.resource)
        self.assertEqual("pass", opts.password)
        self.assertEqual(["conf1", "conf2"], opts.conferences)

    def testBotOptionsLong(self):
        args = """--database-file file.db
                  --jid user@server/resource --password pass
                  --user user2 --server server2 --port 1234
                  --secure 1
                  --join-conference conf1 --join-conference conf2
                  --proxy-host proxy --proxy-port 5678
                  --proxy-user puser --proxy-password ppass"""
        parser = jhc.cli.BotParser()
        opts = parser.getOptions(args.split())
        self.assertEqual("file.db", opts.database)
        self.assertEqual("user@server/resource", opts.jid)
        self.assertEqual("user2", opts.user)
        self.assertEqual("server2", opts.server)
        self.assertEqual("resource", opts.resource)
        self.assertEqual("pass", opts.password)
        self.assertEqual(1234, opts.port)
        self.assertEqual("1", opts.secure)
        self.assertEqual(["conf1", "conf2"], opts.conferences)
        self.assertEqual("proxy", opts.proxy["host"])
        self.assertEqual(5678, opts.proxy["port"])
        self.assertEqual("puser", opts.proxy["user"])
        self.assertEqual("ppass", opts.proxy["password"])

    def testLoadFromFile(self):
        args = "../etc/jhc-test.cfg"
        parser = jhc.cli.BotParser()
        opts = parser.getOptions(args.split())
        self.assertEqual("jhc@jabber.localdomain/test", opts.jid)
        self.assertEqual("123", opts.password)

    def testFilterFile(self):
        string = "one\n#comment\ntwo"
        parser = jhc.cli.BaseParser()
        string = parser.filterFile(string)
        self.assertEqual(["one", "two"], string.split())

    def testMergeOptions(self):
        options1 = optparse.Values()
        options2 = optparse.Values()
        options1.val1 = "1"
        options2.val1 = "2"
        options1.val2 = ["1"]
        options2.val2 = ["2"]
        parser = jhc.cli.BaseParser()
        parser.parser = optparse.OptionParser()
        parser.parser.add_option("--val1")
        parser.parser.add_option("--val2", action="append")
        options = parser.mergeOptions(options1, options2)
        self.assertEqual("2", options.val1)
        self.assertEqual(["1", "2"], options.val2)

    def testMergeDefaultIntOptions(self):
        options1 = optparse.Values()
        options2 = optparse.Values()
        options1.val1 = 2
        options2.val1 = 1
        parser = jhc.cli.BaseParser()
        parser.parser = optparse.OptionParser()
        parser.parser.add_option("--val1", type="int", default=1)
        options = parser.mergeOptions(options1, options2)
        self.assertEqual(2, options.val1)

    def testParseTime(self):
        timeString = "01:02"
        parser = jhc.cli.BaseParser()
        self.assertEqual((1, 2), parser.parseTime(timeString))

    def testGetParamsFromJid(self):
        options = optparse.Values()
        options.jid = "user@server/resource"
        parser = jhc.cli.BaseParser()
        parser.getParamsFromJid(options)
        self.assertEqual("user", options.user)
        self.assertEqual("server", options.server)
        self.assertEqual("resource", options.resource)

def test_suite():
    return unittest.makeSuite(CliTest)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")